<?php

namespace Schaffen\Gef\Http\Controllers;

use App\Http\Controllers\Controller;
use App\Http\Utils\Response;
use App\Http\Utils\SCHArray;
use Illuminate\Http\Request;

use Schaffen\Gef\Criteria\RequestPrefixCriteria;
use Schaffen\Gef\Http\Requests;
use Prettus\Service\Contracts\ServiceInterface;
use Prettus\Service\Exceptions\ServiceException;
use Schaffen\Gef\Http\Requests\VeiculoCreateRequest;
use Schaffen\Gef\Http\Requests\VeiculoUpdateRequest;
use Schaffen\Gef\Http\Requests\VeiculoIndexRequest;
use Schaffen\Gef\Models\Veiculo;
use Schaffen\Gef\Repositories\VeiculoRepository;
use Schaffen\Gef\Services\VeiculoService;

/**
 * Class VeiculosController.
 *
 * @package namespace Schaffen\Gef\Http\Controllers;
 */
class VeiculosController extends Controller
{
   /**
    * @var VeiculoRepository
    */
   protected $repository;

   /**
    * @var VeiculoService
    */
   protected $service;

   /**
    * VeiculosController constructor.
    *
    * @param VeiculoRepository $repository
    * @param VeiculoService $service
    */
   public function __construct(VeiculoRepository $repository, VeiculoService $service)
   {
      $this->repository = $repository;
      $this->service = $service;
   }

   /**
    * Display a listing of the resource.
    *
    * @return \Illuminate\Http\Response
    */
   public function index(Request $request)
   {


      $this->repository->pushCriteria(new \Schaffen\Gef\Criterias\RequestPrefixCriteria($request, Veiculo::$prefix));
      $veiculos = $this->repository->all();

      $ret['cols'] = array();
      array_push($ret['cols'], ['field' => "codigo", 'title' => "Código",'dataType' => 'id', 'filter' => ['codigo' => "text"], 'sortable' => "codigo", 'show' => true]);
      array_push($ret['cols'], ['field' => "identificador", 'title' => "Identificador",'dataType' => 'text', 'filter' => ['identificador' => "text"], 'sortable' => "identificador", 'show' => true]);
      array_push($ret['cols'], ['field' => "placa", 'title' => "Placa",'dataType' => 'text', 'filter' => ['placa' => "text"], 'sortable' => "placa", 'show' => true]);
      array_push($ret['cols'], ['field' => "ano", 'title' => "Ano",'dataType' => 'number', 'filter' => ['ano' => "number"], 'sortable' => "ano", 'show' => true]);
      array_push($ret['cols'], ['field' => "uf", 'title' => "UF",'dataType' => 'text', 'filter' => ['uf' => "text"], 'sortable' => "uf", 'show' => true]);
      array_push($ret['cols'], ['field' => "municipio", 'title' => "Município",'dataType' => 'text', 'filter' => ['municipio' => "text"], 'sortable' => "municipio", 'show' => true]);
      array_push($ret['cols'], ['field' => "modelo", 'title' => "Modelo",'dataType' => 'text', 'filter' => ['modelo' => "text"], 'sortable' => "modelo", 'show' => true]);
      array_push($ret['cols'], ['field' => "cor", 'title' => "Cor",'dataType' => 'text', 'filter' => ['cor' => "text"], 'sortable' => "cor", 'show' => true]);
      array_push($ret['cols'], ['field' => "chassi", 'title' => "Chassi",'dataType' => 'text', 'filter' => ['chassi' => "text"], 'sortable' => "chassi", 'show' => true]);
      array_push($ret['cols'], ['field' => "renavam", 'title' => "Renavam",'dataType' => 'number', 'filter' => ['renavam' => "number"], 'sortable' => "renavam", 'show' => true]);
      array_push($ret['cols'], ['field' => "km_inicial", 'title' => "Km Inicial",'dataType' => 'number', 'filter' => ['km_inicial' => "number"], 'sortable' => "km_inicial", 'show' => true]);
      array_push($ret['cols'], ['field' => "tipo_combustivel", 'title' => "Tipo combustível",'dataType' => 'text', 'filter' => ['tipo_combustivel' => "text"], 'sortable' => "tipo_combustivel", 'show' => true]);
      array_push($ret['cols'], ['field' => "proprietario", 'title' => "Proprietário",'dataType' => 'text', 'filter' => ['proprietario' => "text"], 'sortable' => "proprietario", 'show' => true]);
      array_push($ret['cols'], ['field' => "seguradora", 'title' => "Seguradora",'dataType' => 'text', 'filter' => ['seguradora' => "text"], 'sortable' => "seguradora", 'show' => true]);

      array_push($ret['cols'], ['field' => "action", 'title' => "Opções", 'dataType' => 'command', 'show' => true]);

      $ret['rows'] = $veiculos['data'];

      return Response::response(true,null, $ret,null);

   }

   /**
    * Store a newly created resource in storage.
    *
    * @param  VeiculoCreateRequest $request
    *
    * @return \Illuminate\Http\Response
    *
    * @throws \Prettus\Service\Exceptions\ServiceException
    */
   public function store(VeiculoCreateRequest $request)
   {
      $response = $this->service->updateOrCreate($request->all());
      return Response::response($response['status'],$response['message'], $response['data'],$response['errors'], $response['http_code']);

   }

   /**
    * Display the specified resource.
    *
    * @param  int $id
    *
    * @return \Illuminate\Http\Response
    */
   public function show($id)
   {

      $this->repository->setPresenter("Schaffen\\Gef\\Presenters\\VeiculoShowPresenter");
      $veiculo = $this->repository->find($id);

         return Response::response(true,null, $veiculo,null);

   }

   /**
    * Show the form for editing the specified resource.
    *
    * @param  int $id
    *
    * @return \Illuminate\Http\Response
    */
   public function edit($id)
   {
      $veiculo = $this->repository->find($id);

      return view('veiculos.edit', compact('veiculo'));
   }

   /**
    * Update the specified resource in storage.
    *
    * @param  VeiculoUpdateRequest $request
    * @param  string $id
    *
    * @return Response
    *
    * @throws \Prettus\Service\Exceptions\ServiceException
    */
   public function update(VeiculoUpdateRequest $request, $id)
   {
      $response = $this->service->updateOrCreate($request->all(),$id);
      return Response::response($response['status'],$response['message'], $response['data'],$response['errors'], $response['http_code']);
   }


   /**
    * Remove the specified resource from storage.
    *
    * @param  int $id
    *
    * @return \Illuminate\Http\Response
    */
   public function destroy($id)
   {
      $deleted = $this->repository->delete($id);

      if (request()->wantsJson()) {

         return response()->json([
            'message' => 'Veiculo deleted.',
            'deleted' => $deleted,
         ]);
      }

      return redirect()->back()->with('message', 'Veiculo deleted.');
   }

   public function combustivel()
   {

      $combustivel = Veiculo::$tipos_combustivel;
      return Response::response(true,null, $combustivel,null);
   }
}
